/*  Example implementation of several algorithms
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* I've put all of the algorithms in the same header because they are
   each only one function. Each still has its own implementation file
   (you only need to link against the implementation of the functions
   that you use */

#ifndef CKE_PROJ_ALGO_H
#define CKE_PROJ_ALGO_H

/* a cmp_func takes two void*'s, compares their contents, and returns:
   -1 if dat1 is less than dat2, 0 if dat1 and dat2 are equal, and
   1 if dat1 is greater than dat2 */
typedef int (*cmp_func) (void* dat1, void* dat2);

/* cke_swap (convieniece macro)
   type is the first arg (it is the type of the arguments)
   a and b are the variables to be swapped */
#define cke_swap(type, a, b) { type temp = b; b = a; a = temp; }


/* SORTING ALGORITHMS */

/* bubble sort algorithm. Sorts data using cmp.  data is assumed to
   have sz-1 elements */

void bubble_sort (void* data[], cmp_func cmp, unsigned int sz);

/* selection sort algorithm. Sorts data using cmp. data is assumed to
   have sz-1 elements */

void selection_sort (void* data[], cmp_func cmp, unsigned int sz);

/* insertion sort algorithm. Sorts data using cmp. data is assumed to
   have sz-1 elements */

void insertion_sort (void* data[], cmp_func cmp, unsigned int sz);

/* merge sort algorithm. Sorts data using cmp. data is assumed to have
   sz-1 elements */

void merge_sort (void* data[], cmp_func cmp, unsigned int sz);

/* quick sort algorithm. Sorts data using cmp. data is assumed to have
   sz-1 elements */

void quick_sort (void* data[], cmp_func cmp, unsigned int sz);

/* SEARCHING ALGORITHMS */

/* linear search algorithm. Searches data for key using cmp. data is
   assumed to have sz-1 elements. Returns a pointer to the data item
   if found, NULL otherwise. */

void* linear_search (void* key, void* data[], cmp_func cmp, unsigned int sz);

/* binary search algorithms. Searches data for key using cmp. Data is
   assumed to have sz-1 elements and be sorted in ascendin order.
   Returns a pointer to the data item if found, NULL otherwise */

void* binary_search (void* key, void* data[], cmp_func cmp, unsigned int sz);

#endif /*  CKE_PROJ_ALGO_H */
