/*  Example implementation of a Quick Sort
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "algorithm.h"

/* _quick_sort does the real work. quick_sort is provided for
   consistency with all of the other algorithms.  */
static void _quick_sort (void* data[], cmp_func cmp, int first, int last);

/* splits an array and returns the split point */
static int split (void* data[], cmp_func cmp, int first, int last);

void
quick_sort (void* data[], cmp_func cmp, unsigned int sz)
{
  _quick_sort (data, cmp, 0, sz - 1);
}

void
_quick_sort (void* data[], cmp_func cmp, int first, int last)
{
  int splitp; /* split point */

  if (first < last)
    {
      splitp = split (data, cmp, first, last);
      
      _quick_sort (data, cmp, first, splitp - 1);
      _quick_sort (data, cmp, splitp + 1, last);
    }
}

int
split (void* data[], cmp_func cmp, int first, int last)
{
  int splitp; /* split point */
  void* splitv; /* split value */
  short int brk; /* used as a boolean (if 1 break)*/
  int save_first = first; /* save first for usage later */

  splitv = data[first];

  ++first;

do
  {
    brk = 0;

    while (!brk)
      if (cmp (data[first], splitv) == 1)
	brk = 1;
      else
	{
	  ++first;
	  brk = ( (first <= last) ? 0 : 1);
	}

    brk = ( (first <= last) ? 0 : 1);

    while (!brk)
      if (cmp (data[last], splitv) != 1) /* <= */
	brk = 1;
      else
	{
	  --last;
	  brk = ( (first <= last) ? 0 : 1);
	}

    if (first < last)
      {
	cke_swap (void*, data[first], data[last]);
	++first;
	--last;
      }
  } while (first <= last);

 splitp = last;

 cke_swap (void*, data[save_first], data[splitp]);
 return splitp;
}
