/*  Example test program of several algorithms and data structures
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* This is version 0.1 of my research project test program. Currently
 * it only tests the algorithms, and does them all in order, not
 * allowing the user to skip any. The tests for each algorithms will
 * be moved into separate functions and GNU getopt will be used to
 * allow the testing of different algorithms indivdually through
 * command line arguments. I will then merge the data structure test
 * programs into the unified test program. The ability to change the
 * size of the test data set will also be added through another
 * command line argument.
 */

#include "algorithm.h"
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

int int_cmp (void*, void*); /* compares ints pointed to by void*s */

/* convinience functions */
static void rand_array (int* data[], size_t len); /* randomizes array */
static void free_array (int* data[], size_t len); /* frees an array */
static void print_array (int* data[], size_t len); /* prints array */
static void separator (const char* text); /* separates test section */

#define ARRAYSIZE 50

int main (int argc, char *argv[])
{
  int* int_array[ARRAYSIZE]; /* will hold ARRAYSIZE integers */
  int i, *t = NULL;
  time_t temp_time;

  srand (time (&temp_time));

  /* BUBBLE SORT */
  separator ("Bubble Sort");
  rand_array (int_array, ARRAYSIZE);

  printf ("Before:\n");
  print_array (int_array, ARRAYSIZE);
  printf ("\n\nAfter:\n");

  bubble_sort (int_array, &int_cmp, ARRAYSIZE);

  print_array (int_array, ARRAYSIZE);

  printf ("\n\n");
  free_array (int_array, ARRAYSIZE);

  /* SELECTION SORT */
  separator ("Selection Sort");
  rand_array (int_array, ARRAYSIZE);

  printf ("Before:\n");
  print_array (int_array, ARRAYSIZE);
  printf ("\n\nAfter:\n");

  selection_sort (int_array, &int_cmp, ARRAYSIZE);

  print_array (int_array, ARRAYSIZE);

  printf ("\n\n");
  free_array (int_array, ARRAYSIZE);

  /* INSERTION SORT */
  separator ("Insertion Sort");
  rand_array (int_array, ARRAYSIZE);

  printf ("Before:\n");
  print_array (int_array, ARRAYSIZE);
  printf ("\n\nAfter:\n");

  insertion_sort (int_array, &int_cmp, ARRAYSIZE);

  print_array (int_array, ARRAYSIZE);

  printf ("\n\n");
  free_array (int_array, ARRAYSIZE);

  /* MERGE SORT */
  separator ("Merge Sort");
  rand_array (int_array, ARRAYSIZE);

  printf ("Before:\n");
  print_array (int_array, ARRAYSIZE);
  printf ("\n\nAfter:\n");

  merge_sort (int_array, &int_cmp, ARRAYSIZE);

  print_array (int_array, ARRAYSIZE);

  printf ("\n\n");
  free_array (int_array, ARRAYSIZE);
  /* QUICK SORT */
  separator ("Quick Sort");
  rand_array (int_array, ARRAYSIZE);

  printf ("Before:\n");
  print_array (int_array, ARRAYSIZE);
  printf ("\n\nAfter:\n");

  quick_sort (int_array, &int_cmp, ARRAYSIZE);

  print_array (int_array, ARRAYSIZE);

  printf ("\n\n");
  free_array (int_array, ARRAYSIZE);

  /* LINEAR SEARCH */
  separator ("Linear Search");
  rand_array (int_array, ARRAYSIZE);
  i = rand () % 100;

  printf ("Array:\n");
  print_array (int_array, ARRAYSIZE);
  do
    {
      i = rand () % 1000;
      printf ("\nSearching for: %4d...", i);
      t = (int *) linear_search ((void*)&i, (void*)int_array, &int_cmp, 
				 ARRAYSIZE);
      if (t == NULL)
	printf ("not found");
    } while (t == NULL);
  printf ("found\n");

  free_array (int_array, ARRAYSIZE);

  /* BINARY SEARCH */
  separator ("Binary Search");
  rand_array (int_array, ARRAYSIZE);

  /* we have to sort the array first. I use merge_sort because it is
     generally the fastest sort (Quick Sort's worst case is
     horrible) */
  merge_sort ((void*)int_array, &int_cmp, ARRAYSIZE);

  printf ("Array:\n");
  print_array (int_array, ARRAYSIZE);

  do
    {
      i = rand () % 1000;
      printf ("\nSearching for: %4d...", i);
      t = (int *) binary_search ((void*)&i, (void*)int_array, &int_cmp, 
				 ARRAYSIZE);
      if (t == NULL)
	printf ("not found");
    } while (t == NULL);

  printf ("found\n");
  return 0;
}

#include <assert.h>
int int_cmp (void* a, void* b)
{
  int *f, *g;
  f = (int *) a;
  g = (int *) b;

  if (*f > *g)
    return 1;
  else if (*f < *g)
    return -1;
  else
    return 0;

}

void rand_array (int* data[], size_t len)
{
  int i;
  int *t;
  for (i=0;i<len;++i)
    {
      t = malloc (sizeof (int));
      *t = rand () % 1000;
      data[i] = t;
    }
}

void free_array (int* data[], size_t len)
{
  int i;
  for (i=0;i<len;++i)
    free (data[i]);
}

void print_array (int* data[], size_t len)
{
  int i;
  for (i=0;i<len;++i)
    {
      if (i % 10 == 0)
	printf ("\n"); /* easier to read if in rows of 10 */
      printf ("%4d ", *data[i]);
    }
}
void separator (const char* text)
{
  int i;
  size_t div;
  char *first, *second;
  printf ("\n");
  for (i=0;i<51;++i)
    printf ("=");

  div = strlen (text) / 2;
  second = &text[div];
  first = malloc (div);
  strncpy (first, text, div);
  first[div] = '\0';

  printf ("\n%25s", first);
  printf ("%-25s\n\n", second);

  free (first);
}
