/*  Binary Tree Data Structure Example
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CKE_PROJ_BINTREE_H
#define _CKE_PROJ_BINTREE_H

/* copy_func: returns a copy of data */
/* cmp_func: returns 0 if equal, -1 if dat1 comes before dat2, 1 if
   dat2 comes before dat1. If 0 is returned, the tree functions will
   discard the data being inserted into the tree */
typedef void*  (*copy_func) (void* data);
typedef int (*cmp_func) (void* dat1, void* dat2);


struct _node /* node in the tree */
{
  struct _node* left;
  struct _node* right;
  void* data;
};

struct _bintree
{
  struct _node* tree;
  copy_func copy;
  cmp_func cmp;
};

typedef struct _bintree bintree;

/* binary tree manipulation functions */

/* 
   +   make_bintree: copy_func cmp_func
   +   copy_func is used to copy data inserted into the tree
   +   cmp_func is used to compare the values of data in the tree
   ---
   +   bintree_insert: bintree* tree, void* data
   +   inserts a copy of data into tree
   ---
   +   bintree_get: bintree* tree, void* data
   +   returns a copy of data if found in tree, NULL otherwise
   ---
   +   destroy_bintree: bintree*
   +   detroys the tree and all data contained within
*/

bintree* make_bintree (copy_func, cmp_func);
void bintree_insert (bintree*, void*);
void* bintree_get (bintree*, void*);
void bintree_remove (bintree*, void*);
void destroy_bintree (bintree*);

#endif /* _CKE_PROJ_BINTREE_H */
