/*  Queue Data Structure Example
    Copyright (C) 2001  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <assert.h> /* assertions */
#include "queue.h"

queue*
make_queue (size_t sz)
{
  queue* temp =  (queue *) malloc (1 * sizeof (queue));
  if (sz == 0) /* user tried to create a null queue */
    {
      temp->data = malloc (100 * sizeof (void**));
      temp->size = 100;
    }
  else
    {
      temp->data = malloc (sz * sizeof (void**)) ;
      temp->size = sz;
    }

  temp->front = -1;
  temp->back = -1;
  return temp;
}

void*
queue_peak (queue* que)
{
  assert (que != NULL && que->front != -1);
  return (que->data[que->front]);
}

void
queue_pop (queue* que)
{
  assert (que != NULL && que->front != -1);
  if (que->front == que->back) /* Queue is now empty */
    {
      que->front = -1;
      que->back = -1;
    }
  ++que->front;
}

void
queue_push (queue* que, void* data)
{
  assert (que != NULL);

  if (que->back == que->size)
    {
      que->data = realloc (que->data, (que->size * 2) * sizeof (void**));
      que->size = que->size * 2;
    }
  que->data[que->back+1] = data;
  ++que->back;
  if (que->front == -1)
    que->front = 0;
}

void
queue_destroy (queue* que)
{
  assert (que != NULL && que->data != NULL);
  free (que->data);
  free (que);
  que = NULL;
}
