;    Queue Data Type - Example Library written in Scheme
;    Copyright (C) 2001  Clinton Ebadi
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

; note: this conflicts slib's queue
; also, this is probably not very efficient
; but, efficiency would munge the code, destroying the clarity of the example
;; requires SRFI-1 (list manipulators)

(use-modules (srfi srfi-1))

; destructive drop
(define-macro drop!
  (let ((lst (gensym)) (i (gensym)))
    (lambda (lst i)
      `(set! ,lst (drop ,lst ,i)))))

(define make-queue
  (lambda ()
    (list )))

(define-macro queue-push! ; push data onto the queue
  (let ((queue (gensym)) (data (gensym)))
    (lambda (queue data)
      `(if (null? ,queue)
	   (set! ,queue (list ,data))
	   (append! ,queue (list ,data))))))

(define-macro queue-pop! ; takes data from front of queue
  (let ((queue (gensym)))
    (lambda (queue)
      `(if (null? ,queue)
	   (throw 'null-queue) ; queue has no data, throw exception
	   (begin
	     (define temp (first ,queue))
	     (drop! ,queue 1) ; drop first element
	     temp ; returns temp
	     )))))

(define queue-peak ; peak at front of queue
  (lambda (queue)
    (if (null? queue)
	(throw 'null-queue) ; queue has no data, throw exception
	(first queue))))